/*
*******************************************************************************
**
**  This device driver was created by Applilet for the V850ES/JG2, V850ES/JJ2
**  32-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002-2006
**  All rights reserved by NEC Electronics Corporation
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	timer.c
**  Abstract :	This file implements a device driver for the timer module
**  APIlib :	V850ESJx2.lib V1.70 [7 Dec. 2006]
**
**  Device :	uPD70F3716
**
**  Compiler :	NEC/CA850
**
*******************************************************************************
*/
/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include	"macrodriver.h"
#include	"timer.h"
/*
*******************************************************************************
**  Constants
*******************************************************************************
*/

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function initializes TMM.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void TMM_Init( void )
{
	TM0CTL0 = 0;			/* stop TMM */
	SetIORBit(TM0EQIC0, 0x40);	/* mask interrupt */
	ClrIORBit(TM0EQIC0, 0x80);	/* clear IF flag */
	
	TM0CTL0 |= 0x01;	//* count clock=fxx/2
	
	TM0CMP0 = 0;
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function starts TMM counter.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void TMM_Start( void )
{	
	TM0EQIF0=0;
	SetIORBit(TM0CTL0, 0x80);	/* start counting */
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function stops TMM counter.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void TMM_Stop( void )
{
	ClrIORBit(TM0CTL0, 0x80);
	TM0EQIF0=0;
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function changes TMM register value.
**
**  Parameters:
**	USHORT :	INTERVAL	ő65,534
**
**  Returns:
**
**-----------------------------------------------------------------------------
*/
void wait_us(unsigned short INTERVAL )
{
	unsigned long tmm;
	
	tmm=INTERVAL*10;
	while(tmm){
		if(tmm < 20) break;
		if(tmm > 60000){	//60000=6000uS=6mS
			TM0CMP0 = 60000-1;
			TM0CE=1;	//TMM_Start();	TM0CTL0.7=1;
			while(TM0EQIF0 == 0) ;
			TM0EQIF0=0;		//荞ݗvMNA
			TM0CE=0;	//TMM_Stop();	TM0CTL0.7=0;
			tmm -= 60000;
		}else{
			TM0CMP0 = tmm-1;
			TM0CE=1;	//TMM_Start();	TM0CTL0.7=1;
			while(TM0EQIF0 == 0) ;
			TM0EQIF0=0;		//荞ݗvMNA
			TM0CE=0;	//TMM_Stop();	TM0CTL0.7=0;
			break;
		}
	}
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function changes TMM register value.
**
**  Parameters:
**	unsigned long :	INTERVAL	ő429,496,729
**
**  Returns:
**
**-----------------------------------------------------------------------------
*/
void wait_usl(unsigned long INTERVAL )
{
	unsigned long tmm;

	tmm=INTERVAL*10;
	while(tmm){
		if(tmm < 20) break;
		if(tmm > 60000){	//60000=6000uS=6mS
			TM0CMP0 = 60000-1;
			TM0CE=1;	//TMM_Start();	TM0CTL0.7=1;
			while(TM0EQIF0 == 0) ;
			TM0EQIF0=0;		//荞ݗvMNA
			TM0CE=0;	//TMM_Stop();	TM0CTL0.7=0;
			tmm -= 60000;
		}else{
			TM0CMP0 = tmm-1;
			TM0CE=1;	//TMM_Start();	TM0CTL0.7=1;
			while(TM0EQIF0 == 0) ;
			TM0EQIF0=0;		//荞ݗvMNA
			TM0CE=0;	//TMM_Stop();	TM0CTL0.7=0;
			break;
		}
	}
}
